////////////////////////////////////////////////////////////////////////////////
// TOpenGL component
//
// Component that allows to deal with the OpenGL library
//
// by Hellix
//    products@hellix.com
//    http://www.hellix.com/
//
//  Copyright 1997-99
////////////////////////////////////////////////////////////////////////////////

#ifndef OpenGLH
#define OpenGLH

////////////////////////////////////////////////////////////////////////////////

#include <vcl\sysutils.hpp>
#include <vcl\controls.hpp>
#include <vcl\classes.hpp>
#include <vcl\forms.hpp>
#include <vcl\graphics.hpp>

////////////////////////////////////////////////////////////////////////////////

#include <gl\gl.h>
#include <gl\glu.h>

////////////////////////////////////////////////////////////////////////////////

enum TGLFlag {f_DOUBLEBUFFER, f_DRAW_TO_BITMAP, f_DRAW_TO_WINDOW,
              f_GENERIC_ACCELERATED, f_GENERIC_FORMAT, f_NEED_PALETTE,
              f_NEED_SYSTEM_PALETTE, f_SUPPORT_GDI, f_SUPPORT_OPENGL, f_STEREO,
              f_SWAP_COPY, f_SWAP_EXCHANGE, f_SWAP_LAYER_BUFFERS};

typedef Set<TGLFlag, f_DOUBLEBUFFER, f_SWAP_LAYER_BUFFERS> TGLFlags;

enum TGLPixelType {pt_TYPE_RGBA, pt_TYPE_COLORINDEX};

enum TGLPrintScale {ps_NONE, ps_PROPORTIONAL, ps_PRINT_TO_FIT};

class PACKAGE TOpenGL: public TCustomPanel
{
   private:
      // Private representation of published properties

      HDC DC;
      HGLRC GLContext;

      // Properties used for the format of the pixel (cf. the structure
      // PIXELFORMATDESCRIPTION in "WinGDI.h")

      bool FColorUsedForBackground;
      short FGLColorBits;
      short FGLDepthBits;
      TGLFlags FGLFlags;
      TGLPixelType FGLPixelType;
      int FPixelsPerInch;
      TGLPrintScale FPrintScale;

      // Properties used for internal purpose

      bool NeedToSetControlPixelFormat;

      DWORD ConvertedFlags;
      BYTE ConvertedPixelType;

      HPALETTE Palette;

      TColor OrigColor;
      TColor OrigGLColor;

      // Methods to modify the different published properties

      void __fastcall SetColorUsedForBackground(const bool aValue);
      void __fastcall SetGLColorBits(const short aValue);
      void __fastcall SetGLDepthBits(const short aValue);
      void __fastcall SetGLFlags(const TGLFlags aValue);
      void __fastcall SetGLPixelType(const TGLPixelType aValue);
      inline int __fastcall GetPixelsPerInch() { return (!FPixelsPerInch)?Screen->PixelsPerInch:FPixelsPerInch; }
      void __fastcall SetPixelsPerInch(const int aValue);
      void __fastcall SetPrintScale(const TGLPrintScale aValue);

      // Methods used for internal purpose

      void __fastcall SetControlPixelFormat();
      void __fastcall CreateGLContext();

      void __fastcall ConvertFlags();
      void __fastcall ConvertPixelType();

   protected:
      // Inherited methods

      virtual void __fastcall Paint();
      __declspec(dynamic) void __fastcall Resize();

      virtual void __fastcall CreateWnd();

      // Methods handling different messages (from Windows or not)

      void __fastcall WMCreate(TWMCreate &aMessage);
      void __fastcall WMDestroy(TWMDestroy &aMessage);

      // Special internal events

      TNotifyEvent FOnGLPaint;   // User's routine for GL paint
      TNotifyEvent FOnGLInit;   // User's routine for GL initialisation

   public:
      // Constructor & Destructor

      virtual __fastcall TOpenGL(TComponent *aOwner);

      Graphics::TBitmap* __fastcall GetOpenGLBmp();
      inline void __fastcall ReleaseOpenGLBmp(Graphics::TBitmap *aOpenGLBmp) { if (aOpenGLBmp) delete aOpenGLBmp; }
      void __fastcall SaveToFile(const String &aFileName);
      void __fastcall Print();

      HDC __fastcall GetDC() { return DC; }
      HGLRC __fastcall GetGLContext() { return GLContext; }
      HPALETTE __fastcall GetPalette() { return Palette; }

   __published:
      // Published properties

      __property Align;
      __property Color;
      __property DragCursor;
      __property DragMode;
      __property Enabled;
      __property bool ColorUsedForBackground = {read=FColorUsedForBackground, write=SetColorUsedForBackground, default=false};
      __property short GLColorBits = {read=FGLColorBits, write=SetGLColorBits, default=32};
      __property short GLDepthBits = {read=FGLDepthBits, write=SetGLDepthBits, default=32};
      __property TGLFlags GLFlags = {read=FGLFlags, write=SetGLFlags, nodefault};
      __property TGLPixelType GLPixelType = {read=FGLPixelType, write=SetGLPixelType, default=pt_TYPE_RGBA};
      __property ParentColor;
      __property ParentFont;
      __property ParentShowHint;
      __property int PixelsPerInch = {read=GetPixelsPerInch, write=SetPixelsPerInch, nodefault};
      __property PopupMenu;
      __property TGLPrintScale PrintScale = {read=FPrintScale, write=SetPrintScale, default=ps_PROPORTIONAL};
      __property ShowHint;
      __property TabOrder;
      __property TabStop;
      __property Visible;

      // Available events

      __property OnClick;
      __property OnDblClick;
      __property OnDragDrop;
      __property OnDragOver;
      __property OnEndDrag;
      __property OnEnter;
      __property OnExit;
      __property OnKeyDown;
      __property OnKeyPress;
      __property OnKeyUp;
      __property OnMouseDown;
      __property OnMouseMove;
      __property OnMouseUp;
      __property OnMouseWheel;
      __property OnMouseWheelDown;
      __property OnMouseWheelUp;
      __property OnResize;
      __property OnStartDrag;

      // OpenGL events

      __property TNotifyEvent OnGLPaint = {read=FOnGLPaint, write=FOnGLPaint};
      __property TNotifyEvent OnGLInit = {read=FOnGLInit, write=FOnGLInit};

   BEGIN_MESSAGE_MAP
      MESSAGE_HANDLER(WM_CREATE, TWMCreate, WMCreate)
      MESSAGE_HANDLER(WM_DESTROY, TWMDestroy, WMDestroy)
   END_MESSAGE_MAP(TCustomPanel)
};

////////////////////////////////////////////////////////////////////////////////

#endif

// End of file
////////////////////////////////////////////////////////////////////////////////

